***********************************************************************
*     file crosssection.F                                             *
*---------------------------------------------------------------------*
*     03.03.11  Stefan Kallweit     last changed  03.03.11            *
***********************************************************************


***********************************************************************
      subroutine crosssection(k,x1,x2,generator,phasespace,em,sp,
     &    switch)
***********************************************************************
*     calculation of cross sections                                   *
*     selection, which process to calculate                           *
*---------------------------------------------------------------------*
*     03.03.11  Stefan Kallweit     last changed  03.03.11            *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"

      integer em,sp
      integer generator,phasespace,switch
      real*8 k(maxe,0:3)
      real*8 x1,x2

      integer selprocess
      common/selprocess/selprocess

      if (selprocess.eq.0)then
         call vbfh_crosssection(k,x1,x2,generator,phasespace,em,sp,
     &    switch)
      elseif (selprocess.ge.1) then
         call whzh_crosssection(k,x1,x2,generator,phasespace,em,sp,
     &    switch)
c      elseif (selprocess.eq.1) then
c         call crosssection_Hln(k,x1,x2,generator,phasespace,em,sp,
c     &    switch)
c      elseif (selprocess.eq.2) then
c         call crosssection_Hnl(k,x1,x2,generator,phasespace,em,sp,
c     &    switch)
c      elseif (selprocess.eq.3) then
c         call crosssection_Hll(k,x1,x2,generator,phasespace,em,sp,
c     &    switch)
c      elseif (selprocess.eq.4) then
c         call crosssection_Hnn(k,x1,x2,generator,phasespace,em,sp,
c     &    switch)
      endif

      end

***********************************************************************
      subroutine onsproj(k,kons,belthr,lnext,switch)
***********************************************************************
*     on-shell projection of external momenta                         *
*     projects Higgs momentum on-shell by rescaling other momenta     *
*---------------------------------------------------------------------*
*     04.03.10  Ansgar Denner     last changed  22.03.10              *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"

c local variables
      integer lnext,switch
      real*8 k(6,0:3)
      real*8 kons(6,0:3)
      real*8 scafac,sh,sinc,shin,disc
      logical belthr

      belthr=.false.
      if(shtr.eq.0) then
        do ext=1,lnext
          do vcomp=0,3
            kons(ext,vcomp)=k(ext,vcomp)
          enddo
        enddo
      else if (shtr.ge.1) then
        if(sonsproj.eq.0) then
          do ext=1,lnext
            do vcomp=0,3
              kons(ext,vcomp)=k(ext,vcomp)
            enddo
          enddo
        elseif(sonsproj.eq.1)then
          sh = k(5,0)*k(5,0)-k(5,1)*k(5,1)-k(5,2)*k(5,2)-k(5,3)*k(5,3)
          sinc= (k(1,0)+k(2,0))*(k(1,0)+k(2,0))
     &        -(k(1,1)+k(2,1))*(k(1,1)+k(2,1))
     &        -(k(1,2)+k(2,2))*(k(1,2)+k(2,2))
     &        -(k(1,3)+k(2,3))*(k(1,3)+k(2,3))
          shin= (k(1,0)+k(2,0))*k(5,0)
     &        -(k(1,1)+k(2,1))*k(5,1)
     &        -(k(1,2)+k(2,2))*k(5,2)
     &        -(k(1,3)+k(2,3))*k(5,3)
          disc = shin*shin-sinc*sh+mcm(25)*mcm(25)*(sinc-2d0*shin+sh)

          if(sinc.lt.mcm(25)*mcm(25).or.disc.lt.0d0) then
            belthr=.true.
            return
          else
            scafac= (sinc-mcm(25)*mcm(25))/(sinc-shin+sqrt(disc))
c          else
c            scafac= (sinc-shin+sqrt(disc))/(sinc-2d0*shin+sh)
          endif


          if(lnext.eq.5)then
            do vcomp=0,3
              kons(1,vcomp)=k(1,vcomp)
              kons(2,vcomp)=k(2,vcomp)
              kons(3,vcomp)=k(3,vcomp)*scafac
              kons(4,vcomp)=k(4,vcomp)*scafac
              kons(5,vcomp)=kons(1,vcomp)+kons(2,vcomp)
     &            -kons(3,vcomp)-kons(4,vcomp)
            enddo
          elseif(lnext.eq.6)then
            do vcomp=0,3
              kons(1,vcomp)=k(1,vcomp)
              kons(2,vcomp)=k(2,vcomp)
              kons(3,vcomp)=k(3,vcomp)*scafac
              kons(4,vcomp)=k(4,vcomp)*scafac
              kons(6,vcomp)=k(6,vcomp)*scafac
              kons(5,vcomp)=kons(1,vcomp)+kons(2,vcomp)
     &            -kons(3,vcomp)-kons(4,vcomp)-kons(6,vcomp)
            enddo
          else
            write(*,*) 'onshprj: wrong number of external particles:',
     &          lnext
          endif
        elseif(sonsproj.eq.2)then
          sh = k(5,0)*k(5,0)-k(5,1)*k(5,1)-k(5,2)*k(5,2)-k(5,3)*k(5,3)
          scafac= mcm(25)/sqrt(sh)
          do ext=1,lnext
            do vcomp=0,3
              kons(ext,vcomp)=k(ext,vcomp)*scafac
            enddo
          enddo
        endif
      else
        write(*,*) 'onsproj : unknown input shtr = ',shtr
        stop
      endif

      return

      sh = k(5,0)*k(5,0)-k(5,1)*k(5,1)-k(5,2)*k(5,2)-k(5,3)*k(5,3)
      write(*,*) 'onsproj',sonsproj
      write(*,*) 'onsproj:  sh     = ',sh
      write(*,*) 'onsproj:  shproj = ',
     &    kons(5,0)*kons(5,0)-kons(5,1)*kons(5,1)
     &    -kons(5,2)*kons(5,2)-kons(5,3)*kons(5,3),
     &    mcm(25)*mcm(25)
      call printmomenta(0d0,0d0,k,lnext)
      call printmomenta(0d0,0d0,kons,lnext)

      end
