***********************************************************************
*     file mcmodel.F                                                  *
*     initializes model dependent part of Monte Carlo                 *
*---------------------------------------------------------------------*
*     30.01.06  Ansgar Denner     last changed  23.11.09              *
***********************************************************************
*
*        power changed  from .8 to .9  !!!!!!!!!!!!
*
*
***********************************************************************
      subroutine init_subprocesses(lenergy,sppmin,generator,
     &    lnext,lsmodel,lsincludecuts,lssub)
***********************************************************************
*     determines partonic processes for generator and                 *
*     initializes generator                                           *
*---------------------------------------------------------------------*
*     20.03.06  Ansgar Denner     last changed  20.03.06              *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"

c      local variables
      real*8  lenergy,lsppmin,sppmin
      integer generator,lnext,lsmodel,lsincludecuts,lssub
      integer ncont(maxe),cont(maxe+1)
      integer parton(maxe,maxpart)
      integer i1,flag,pproc

c ncont(ext) is number of partons in external particle ext
c parton(ext,cont) gives hepnumber of parton(cont) in ext
c nproc: number of partonic processes

c options
c     lnoutgen=0                  no output for generator
c     lnoutgen=1                  prints summary of channels
c     lnoutgen=2                  prints all channels
c     lnoutgen=3                  prints also existing partonic processes
c                                 that need no new generators
c     lnoutgen=5                  prints also nonexisting partonic processes

      if(lnoutgen.gt.0)then
      write(nout,50) generator,processname(generator),lnext
 50   format(/' Initialize subprocesses for generator',i2,' :'
     &       /' Process:  ',a60,
     &       /' External particles:',i2)
      endif

c external particles
      nexternal(generator) = lnext

c fixing naux in init_mcmodel is too late
      naux = 100

c setting mapping identifiers to false
c    taken out from generator since this is called several times
      do ext = 0,lnext
        do ext1 = 0,lnext
          do ext2 = 0,lnext
            lmap(ext,ext1,ext2,generator)=.false.
          enddo
        enddo
      enddo

c determine number of partons  ncont(ext) in each external particle
c based on partcont(expart,mcpart)
      nproc(generator) = 1
      do ext=1,nexternal(generator)
        if (abs(hepnum(ext,generator)).lt.ncomp) then
          ncont(ext)=1
          parton(ext,1) = hepnum(ext,generator)
          if(lnoutgen.gt.0) then
          write(nout,100) ext,parton(ext,1)
 100      format(' External particle',i2,' (',i3,')',' is elementary')
          endif
        else
c         determine particle content of composite particle
          ncont(ext)=0
          do mcpart=-naux+1,naux-1
c          write(nout,*) mcpart,hepnum(ext,generator),partcont(hepnum(ext,generator),mcpart)
            if (partcont(abs(hepnum(ext,generator)),mcpart)) then
             ncont(ext)=ncont(ext)+1
             parton(ext,ncont(ext)) = mcpart
            endif
          enddo
          if(lnoutgen.gt.0)then
          write(nout,200) ext,hepnum(ext,generator),ncont(ext),
     &        (parton(ext,i1),i1=1,ncont(ext))
 200      format(' External particle',i2,' (',i3,') contains',
     &        i2,' partons: ',20(i3:' , '))
          endif
          if (ncont(ext).eq.0) then
            write(nout,90) hepnum(ext,generator)
 90         format(' Composite particle ',i3,' contains no partons')
            stop
          endif
        endif
        nproc(generator) = nproc(generator)*ncont(ext)
      enddo

      do ext=1,nexternal(generator)
         cont(ext) = 1
      enddo
      cont(1) = 0

      if(lnoutgen.gt.0)then
      write(nout,*)
      write(nout,*) 'Initializing generator for partonic processes'
      endif
      sppmin = lenergy*lenergy
      proc = 0
      nchannel(generator) = 0
      do pproc = 1,nproc(generator)
        proc = proc + 1
        cont(1) = cont(1) + 1
        ext = 1
        do while(cont(ext).gt.ncont(ext))
          cont(ext) = 1
          ext = ext+1
          cont(ext) = cont(ext) + 1
        enddo
        do ext=1,nexternal(generator)
          prochepnum(ext,proc,generator) =  parton(ext,cont(ext))
        enddo
c initialize generator for partonic process
        call initgenerator(lenergy,lsppmin,prochepnum(1,proc,generator),
     *      generator,lnext,lsmodel,lsincludecuts,lssub,flag)
c        write(*,*) 'init_subprocesses: flag = ',flag

c same minimal energy for all subprocesses!!
        sppmin = min(lsppmin,sppmin)
        if (flag.eq.0) then
c partonic process does not exist
          if(nout.ne.0.and.lnoutgen.ge.5) then
            write(*,500) proc,prochepnum(1,proc,generator),
     &          prochepnum(2,proc,generator),
     &          prochepnum(3,proc,generator),
     &          (prochepnum(ext,proc,generator),
     &          ext=4,nexternal(generator))
          endif
 500        format(/' partonic process does not exist: ',
     &          i3,':',i3,' + ',i3,' -> ',10(i3:' + '))
            proc = proc - 1
          else if (flag.eq.-1) then
c partonic process exists and does not need new generator
          if(nout.ne.0.and.lnoutgen.ge.3)then
            write(*,510) proc,prochepnum(1,proc,generator),
     &          prochepnum(2,proc,generator),
     &          prochepnum(3,proc,generator),
     &          (prochepnum(ext,proc,generator),
     &          ext=4,nexternal(generator))
          endif
 510      format(/' partonic process without new generator:  ',
     &        i3,':',i3,' + ',i3,' -> ',10(i3:' + '))
        else if (flag.eq.1) then
c partonic process exists and needs new generator
          if(nout.ne.0.and.lnoutgen.ge.2)then
            write(*,520) proc,prochepnum(1,proc,generator),
     &          prochepnum(2,proc,generator),
     &          prochepnum(3,proc,generator),
     &          (prochepnum(ext,proc,generator),
     &          ext=4,nexternal(generator))
          endif
 520        format(/' partonic process with new generator:  ',
     &          i3,':',i3,' + ',i3,' -> ',10(i3:' + '))
        else
          write(*,*) ' init_subprocesses: ',
     &        ' flag not defined: flag = ',flag
        endif
      enddo
      nproc(generator) = proc

      if(nout.ne.0.and.lnoutgen.ge.1)then
c output information on generator
        write(nout,'(a)')' '
        write(nout,'(a23,i2)')' Phase-space generator ',generator
        write(nout,'(" Number of partonic processes  =",i6)')
     *    nproc(generator)
        write(nout,'(" Number of channels            =",i6)')
     *    nchannel(generator)
        write(nout,'(" Calculation of invariants     =",i6)')
     *    maxinv(generator)
        write(nout,'(" Calculation of 2->2 processes =",i6)')
     *    maxprocess(generator)
        write(nout,'(" Calculation of 1->2 decays    =",i6)')
     *    maxdecay(generator)
      endif



      end


***********************************************************************
      subroutine init_particles()
***********************************************************************
*     initializes names of particles for input                        *
*     and hadron content                                              *
*---------------------------------------------------------------------*
*     31.01.06  Ansgar Denner     last changed  31.01.06              *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"

c     local variables
      character*3 madname(-maxv:maxv)

* type=1  neutrino
* type=2  charged lepton
* type=3  quark
* type=4  gauge boson
* type=5  Higgs boson

      hepname(0)="     "
      class(0)="     "
      do ipart=1,maxip
        hepname(ipart)="     "
        hepname(-ipart)="     "
        class(ipart)="     "
        class(-ipart)="     "
        mcm(ipart)=0D0
        mcw(ipart)=0D0
        do mcpart=-maxv,maxv
          partcont(ipart,mcpart) = .false.
        enddo
      enddo

c particle names
      hepname(1)='d'
      hepname(2)='u'
      hepname(3)='s'
      hepname(4)='c'
      hepname(5)='b'
      hepname(6)='t'
      do ipart=1,6
        class(ipart)='quark'
        class(-ipart)='quark'
        hepname(-ipart)='anti-'//hepname(ipart)
        family(ipart)=(ipart+1)/2
        family(-ipart)=family(ipart)
      enddo
      hepname(11)='e'
      hepname(12)='nu_e'
      hepname(13)='mu'
      hepname(14)='nu_mu'
      hepname(15)='tau'
      hepname(16)='nu_tau'
      do ipart=11,16
        if(mod(ipart,2).ne.0)then
          class(ipart)='charged lepton'
          class(-ipart)='charged lepton'
        endif
        family(ipart)=(ipart-9)/2
        family(-ipart)=family(ipart)
        hepname(-ipart)='anti-'//hepname(ipart)
      enddo
      hepname(22)='photon'
      hepname(23)='Z'
      hepname(24)='W-'
      hepname(-24)='W+'
      hepname(25)='h0'
      if(smodel.eq.2.or.smodel.eq.12.or.smodel.eq.125)then
        hepname(26)='gluon'
      endif
c higgs singlet
      if(smodel.eq.15.or.smodel.eq.125)then
        hepname(29)='s0'
      endif
c supersymmetric particles
      if(smodel.eq.3.or.smodel.eq.34)then
        hepname(31)='sd_1'
        hepname(32)='su_1'
        hepname(33)='ss_1'
        hepname(34)='sc_1'
        hepname(35)='sb_1'
        hepname(36)='st_1'
        do ipart=31,36
          class(ipart)='charged squark'
          class(-ipart)='charged squark'
          family(ipart)=(ipart-29)/2
          family(-ipart)=family(ipart)
          hepname(-ipart)='anti-'//hepname(ipart)
        enddo
        hepname(41)='sd_2'
        hepname(42)='su_2'
        hepname(43)='ss_2'
        hepname(44)='sc_2'
        hepname(45)='sb_2'
        hepname(46)='st_2'
        do ipart=41,46
          class(ipart)='charged squark'
          class(-ipart)='charged squark'
          family(ipart)=(ipart-39)/2
          family(-ipart)=family(ipart)
          hepname(-ipart)='anti-'//hepname(ipart)
        enddo
        hepname(51)='se_1'
        hepname(52)='snu_e1'
        hepname(53)='smu_1'
        hepname(54)='snu_mu1'
        hepname(55)='stau_1'
        hepname(56)='snu_tau1'
        do ipart=51,56
          if(mod(ipart,2).ne.0)then
            class(ipart)='charged slepton'
            class(-ipart)='charged slepton'
          endif
          family(ipart)=(ipart-49)/2
          family(-ipart)=family(ipart)
          hepname(-ipart)='anti-'//hepname(ipart)
        enddo
        hepname(61)='se_2'
        hepname(63)='smu_2'
        hepname(65)='stau_2'
        do ipart=61,66,2
          if(mod(ipart,2).ne.0)then
            class(ipart)='charged slepton'
            class(-ipart)='charged slepton'
          endif
          family(ipart)=(ipart-59)/2
          family(-ipart)=family(ipart)
          hepname(-ipart)='anti-'//hepname(ipart)
        enddo
        hepname(71)='chi0_1'
        hepname(72)='chi0_2'
        hepname(73)='chi0_3'
        hepname(74)='chi0_4'
        hepname(75)='chi-_1'
        hepname(-75)='chi+_1'
        hepname(76)='chi-_2 '
        hepname(-76)='chi+_2 '
        hepname(81)='A0'
        hepname(82)='H0'
        hepname(83)='H-'
        hepname(-83)='H+'
      endif
      if(smodel.eq.34.or.smodel.eq.4)then
        hepname(84)='gluino'
      endif

c particle names for MadgraphII

* composite particles
      ncomp = 100

* proton
      hepname(101)="proton"
      class(101)="proton"
      partcont(101,1) = .true.
      partcont(101,2) = .true.
      partcont(101,3) = .true.
      partcont(101,4) = .true.
      partcont(101,-1) = .true.
      partcont(101,-2) = .true.
      partcont(101,-3) = .true.
      partcont(101,-4) = .true.

      partcont(101,3) = .false.
      partcont(101,4) = .false.
      partcont(101,-3) = .false.
      partcont(101,-4) = .false.
c      partcont(101,1) = .false.
c      partcont(101,-1) = .false.
c      partcont(101,2) = .false.
c      partcont(101,-2) = .false.

* anti-proton
      hepname(-101)="anti-proton"
      class(-101)="proton"

* jet
      hepname(102)="jet"
      class(102)="jet"
      partcont(102,1) = .true.
      partcont(102,2) = .true.
      partcont(102,3) = .true.
      partcont(102,4) = .true.
      partcont(102,-1) = .true.
      partcont(102,-2) = .true.
      partcont(102,-3) = .true.
      partcont(102,-4) = .true.

      partcont(102,3) = .false.
      partcont(102,4) = .false.
      partcont(102,-3) = .false.
      partcont(102,-4) = .false.
c      partcont(102,1) = .false.
c      partcont(102,-1) = .false.
c      partcont(102,2) = .false.
c      partcont(102,-2) = .false.


c put masses and widths to zero for light fermions

c check with Madgraph
c      do ipart=0,maxv
c        width(ipart)=0d0
c        if(ipart.ne.25)mass(ipart)=0d0
c        power(ipart)=a+1d-5*ipart
c      enddo
c mapping

      end

***********************************************************************
      subroutine init_mcmodel(lsmap)
***********************************************************************
*     initializes parameters of particles for monte carlo generator   *
*---------------------------------------------------------------------*
*     07.02.06  Ansgar Denner     last changed  07.02.06              *
*     adapted from Markus Roth                                        *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"

c local variables
      character*9 particle(maxv)
      character*80 vertices
      logical vertex,schannel
      integer lsmap,mcauxpart,ivert,ifam

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     initialize particles                                        c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
*     genuine particles: mcpart=1,..,naux-1                       *
*     auxiliary particles for 4 vertices: naux, ..., nmap-1       *
*     auxiliary particles for extra mappings of unstable particles*
*                                         nmap, ..., maxv         *
*******************************************************************
c generic particle names, see also subroutine vertexg
      do mcpart=1,maxv
        particle(mcpart)=' '
      enddo

c name, generic name, anti-field, family
      particle(1)='dq dq y 1'
      particle(2)='uq uq y 1'
      particle(3)='sq dq y 2'
      particle(4)='cq uq y 2'
      particle(5)='bq dq y 3'
      particle(6)='tq uq y 3'
      particle(11)='el el y 4'
      particle(12)='ne ne y 4'
      particle(13)='mu el y 5'
      particle(14)='nm ne y 5'
      particle(15)='ta el y 6'
      particle(16)='nt ne y 6'
      particle(22)='ph ph n  '
      particle(23)='Z0 Z0 n  '
      particle(24)='W- W- y  '
      particle(25)='h0 h0 n  '
      particle(26)='gl gl n  '
      particle(29)='s0 s0 n  '

c supersymmetric particles
      particle(31)='D1 DQ y 1'
      particle(32)='U1 UQ y 1'
      particle(33)='S1 DQ y 2'
      particle(34)='C1 UQ y 2'
      particle(35)='B1 DQ y 3'
      particle(36)='T1 UQ y 3'
      particle(41)='D2 DQ y 1'
      particle(42)='U2 UQ y 1'
      particle(43)='S2 DQ y 2'
      particle(44)='C2 UQ y 2'
      particle(45)='B2 DQ y 3'
      particle(46)='T2 UQ y 3'
      particle(51)='E1 EL y 4'
      particle(52)='NE NE y 4'
      particle(53)='M1 EL y 5'
      particle(54)='NM NE y 5'
      particle(55)='T1 EL y 6'
      particle(56)='NT NE y 6'
      particle(61)='E2 EL y 4'
      particle(63)='M2 EL y 5'
      particle(65)='T2 EL y 6'
      particle(71)='N1 C0 n  '
      particle(72)='N2 C0 n  '
      particle(73)='N3 C0 n  '
      particle(74)='N4 C0 n  '
      particle(70)='C1 C- y  '
      particle(76)='C2 C- y  '
      particle(81)='A0 A0 n  '
      particle(82)='H0 H0 n  '
      particle(83)='H- H- y  '
      particle(84)='GL GL n  '

c auxiliary particles
      naux=100
c naux is also fixed in  init_subprocesses
      particle(101)=' 1  1 y  '
      particle(102)=' 2  2 y  '
      particle(103)=' 3  3 y  '
      particle(104)=' 4  4 y  '
      particle(105)=' 5  5 y  '
      particle(106)=' 6  6 y  '
      particle(107)=' 7  7 y  '
      particle(108)=' 8  8 y  '
      particle(109)=' 9  9 y  '
      particle(110)='10 10 y  '
      particle(111)='11 11 y  '
      particle(112)='12 12 y  '
      particle(113)='13 13 y  '
      particle(114)='14 14 y  '
      particle(115)='15 15 y  '
      particle(116)='16 16 y  '  ! non-standard couplings

c model settings
      if(smodel.eq.1)then        ! sm
        do mcpart=26,84
          particle(mcpart)=' '
        enddo
      elseif(smodel.eq.12   ! sm and qcd
     &      .or.smodel.eq.15.or.smodel.eq.125)then   ! additional singlet
        do mcpart=31,84
          particle(mcpart)=' '
        enddo
      elseif(smodel.eq.2)then    ! qcd
        do mcpart=22,25
          particle(mcpart)=' '
        enddo
        do mcpart=31,84
          particle(mcpart)=' '
        enddo
      elseif(smodel.eq.3)then    ! mssm
        particle(26)=' '
        particle(84)=' '
      elseif(smodel.eq.34)then   ! mssm and sqcd
      elseif(smodel.eq.4)then    ! sqcd
        do mcpart=22,25
          particle(mcpart)=' '
        enddo
        do mcpart=71,83
          particle(mcpart)=' '
        enddo
      endif
      do mcpart=85,100
        particle(mcpart)=' '
      enddo

c additional particles for unstable particles
c to allow for extra flat mappings
      nmap=150
      if(lsmap.eq.2)then
        mcpart=nmap
        do mcpart2=1,naux-1
          if(mcwidth(mcpart2).ne.0d0.and.particle(mcpart2)(1:2).ne.'  ')
     &        then
            particle(mcpart)=particle(mcpart2)
            mcpart=mcpart+1
            if(mcpart.gt.maxv)then
              write(*,'(a)')' initgenerator: maxv too small!'
              stop
            endif
          endif
        enddo
      endif

c setting particles
      gname(0)='   '
      do mcpart=1,maxv
        do ifam=1,6
          mcfamily(mcpart,ifam)=0
        enddo
        if(particle(mcpart)(9:9).eq.'1')mcfamily(mcpart,1)=1
        if(particle(mcpart)(9:9).eq.'2')mcfamily(mcpart,2)=1
        if(particle(mcpart)(9:9).eq.'3')mcfamily(mcpart,3)=1
        if(particle(mcpart)(9:9).eq.'4')mcfamily(mcpart,4)=1
        if(particle(mcpart)(9:9).eq.'5')mcfamily(mcpart,5)=1
        if(particle(mcpart)(9:9).eq.'6')mcfamily(mcpart,6)=1
        do ifam=1,6
          mcfamily(-mcpart,ifam)=-mcfamily(mcpart,ifam)
        enddo
        pname(mcpart)=particle(mcpart)(1:3)
        pname(-mcpart)=pname(mcpart)
        gname(mcpart)=particle(mcpart)(4:6)
        gname(-mcpart)=gname(mcpart)
        if(particle(mcpart)(7:7).eq.'y')then
          pname(-mcpart)(3:3)='~'
          gname(-mcpart)(3:3)='~'
        endif
      enddo

c output of couplings
      if(nout.ne.0.and.noutgen.eq.2)then
        schannel = .true.
        write(nout,'(a)')' '
        write(nout,'(a)')' Couplings:'
        ivert=1
        vertices=' '
        do mcpart1=-naux+1,naux-1
        do mcpart2=mcpart1+1,naux
        do mcpart3=mcpart2+1,naux
        if(vertex(mcpart1,mcpart2,mcpart3,schannel).and.
     *    (gname(mcpart1)(3:3).ne.' '.or.mcpart1.gt.0).and.
     *    (gname(mcpart2)(3:3).ne.' '.or.mcpart2.gt.0).and.
     *    (gname(mcpart3)(3:3).ne.' '.or.mcpart3.gt.0))then
            vertices=vertices(1:ivert)//
     *        ' ('//pname(mcpart1)//','//pname(mcpart2)//','//
     &        pname(mcpart3)//')'
            ivert=ivert+14
            if(ivert.gt.60)then
              write(nout,'(a)')vertices
              vertices=' '
              ivert=1
            endif
        endif
        enddo
        enddo
        enddo
        write(nout,'(a)')vertices
        ivert=1
        vertices=' '
        do mcauxpart=naux,maxv
        do mcpart1=-naux+1,naux-1
        do mcpart2=mcpart1+1,naux-1
        schannel=.true.
        if(vertex(mcauxpart,mcpart1,mcpart3,schannel).and.
     *    (gname(mcpart1)(3:3).ne.' '.or.mcpart1.gt.0).and.
     *    (gname(mcpart2)(3:3).ne.' '.or.mcpart1.gt.0))then
          do mcpart3=-naux+1,naux-1
          do mcpart4=mcpart3+1,naux-1
          if(vertex(-mcauxpart,mcpart3,mcpart4,schannel).and.
     *      (gname(mcpart3)(3:3).ne.' '.or.mcpart3.gt.0).and.
     *      (gname(mcpart4)(3:3).ne.' '.or.mcpart4.gt.0))then
            vertices=vertices(1:ivert)//' ('//pname(mcpart1)//','//
     *          pname(mcpart2)//','//pname(mcpart3)//','//
     *          pname(mcpart4)//')'
            ivert=ivert+18
            if(ivert.gt.60)then
              write(nout,'(a)')vertices
              vertices=' '
              ivert=1
            endif
          endif
          enddo
          enddo
        endif
        enddo
        enddo
        enddo
        write(nout,'(a)')vertices
        write(nout,'(a)')' '
        write(nout,'(a)')' Channels:'
      endif

c  set power that controls the mapping of the propagators
      do mcpart=0,maxv
        if(mcmass(mcpart).eq.0d0)then
c          power(mcpart)=0.8d0
c changed 29.10. AD
          power(mcpart)=powerprop
          power(mcpart)=0.9d0
        else
          power(mcpart)=2d0
        endif
        if(mcwidth(mcpart).gt.0d0)power(mcpart)=0d0
        if(mcpart.ge.naux)then
          mcmass(mcpart)=0d0
          mcwidth(mcpart)=0d0
          power(mcpart)=0d0
        endif
      enddo

      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     generic 3-particle vertex                                   c
c                                                                 c
c     written by Markus Roth                                      c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function vertexg(i1,i2,i3,schannel)
***********************************************************************
*     valid vertices for generation of MC channels                    *
*     schannel used to avoid double counting from                     *
*     4-vertices with 4 identical particles                           *
*---------------------------------------------------------------------*
*     27.01.06  Ansgar Denner     last changed  27.03.07              *
*     taken from Markus Roth                                          *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"

c local variables
      integer i1,i2,i3
      character*4 p1,p2,p3,g1,g2,g3
      logical vertexg,nonstandardcoup,schannel

      vertexg=.true.
      p1=pname(i1)
      p2=pname(i2)
      p3=pname(i3)
      g1=gname(i1)
      g2=gname(i2)
      g3=gname(i3)

c EWSM couplings
      if(smodel.eq.1.or.smodel.eq.12.or.smodel.eq.3.or.smodel.eq.34
     &    .or.smodel.eq.125.or.smodel.eq.15)then
c 2 leptons - higgs
        if(light(i2).eq.0)then
          if(g1.eq.'el '.and.g2.eq.'el~'.and.g3.eq.'h0 ')return
        endif
c 2 quarks - higgs
        if(light(i1).eq.0.and.light(i2).eq.0)then
          if(g1.eq.'dq '.and.g2.eq.'dq~'.and.g3.eq.'h0 ')return
          if(g1.eq.'uq '.and.g2.eq.'uq~'.and.g3.eq.'h0 ')return
        endif
c 2 leptons - gauge boson
        if(g1.eq.'el '.and.g2.eq.'el~'.and.g3.eq.'ph ')return
        if(g1.eq.'ne '.and.g2.eq.'ne~'.and.g3.eq.'Z0 ')return
        if(g1.eq.'el '.and.g2.eq.'el~'.and.g3.eq.'Z0 ')return
        if(g1.eq.'ne '.and.g2.eq.'el~'.and.g3.eq.'W- ')return
        if(g1.eq.'ne~'.and.g2.eq.'el '.and.g3.eq.'W-~')return
c 2 quarks - gauge boson
        if(g1.eq.'uq '.and.g2.eq.'uq~'.and.g3.eq.'ph ')return
        if(g1.eq.'dq '.and.g2.eq.'dq~'.and.g3.eq.'ph ')return
        if(g1.eq.'uq '.and.g2.eq.'uq~'.and.g3.eq.'Z0 ')return
        if(g1.eq.'dq '.and.g2.eq.'dq~'.and.g3.eq.'Z0 ')return
        if(g1.eq.'uq '.and.g2.eq.'dq~'.and.g3.eq.'W- ')return
        if(g1.eq.'uq~'.and.g2.eq.'dq '.and.g3.eq.'W-~')return
c 3 higgs
        if(g1.eq.'h0 '.and.g2.eq.'h0 '.and.g3.eq.'h0 ')return
c higgs - 2 gauge bosons
        if(g1.eq.'h0 '.and.g2.eq.'Z0 '.and.g3.eq.'Z0 ')return
        if(g1.eq.'h0 '.and.g2.eq.'W- '.and.g3.eq.'W-~')return
c 3 gauge bosons
        if(g1.eq.'ph '.and.g2.eq.'W- '.and.g3.eq.'W-~')return
        if(g1.eq.'Z0 '.and.g2.eq.'W- '.and.g3.eq.'W-~')return
c 4 particle vertices
        if(g1.eq.'ph '.and.g2.eq.'W-~'.and.g3.eq.' 5 ')return
        if(g1.eq.'Z0 '.and.g2.eq.'W-~'.and.g3.eq.' 5 ')return
        if(g1.eq.'ph '.and.g2.eq.'W- '.and.g3.eq.' 5~')return
        if(g1.eq.'Z0 '.and.g2.eq.'W- '.and.g3.eq.' 5~')return
        if(g1.eq.'ph '.and.g2.eq.'W- '.and.g3.eq.' 6~')return
        if(g1.eq.'Z0 '.and.g2.eq.'W- '.and.g3.eq.' 6~')return
        if(g1.eq.'ph~'.and.g2.eq.'W-~'.and.g3.eq.' 6 ')return
        if(g1.eq.'Z0 '.and.g2.eq.'W-~'.and.g3.eq.' 6 ')return
        if(g1.eq.'W+ '.and.g2.eq.'W- '.and.g3.eq.' 7 ')return
        if(g1.eq.'W+ '.and.g2.eq.'W- '.and.g3.eq.' 7~')return
        if(g1.eq.'ph '.and.g2.eq.'ph '.and.g3.eq.' 7~')return
        if(g1.eq.'ph '.and.g2.eq.'Z0 '.and.g3.eq.' 7~')return
        if(g1.eq.'Z0 '.and.g2.eq.'Z0 '.and.g3.eq.' 7~')return
        if(g1.eq.'h0 '.and.g2.eq.'h0 '.and.g3.eq.' 7~')return
        if(schannel)then
          if(g1.eq.'h0 '.and.g2.eq.'h0 '.and.g3.eq.' 8 ')return
          if(g1.eq.'h0 '.and.g2.eq.'h0 '.and.g3.eq.' 8~')return
c added AD 27.03.07
          if(g1.eq.'Z0 '.and.g2.eq.'Z0 '.and.g3.eq.' 8~')return
        endif
      endif

c Higgs singlet
      if(smodel.eq.15.or.smodel.eq.125)then
        if(g1.eq.'h0 '.and.g2.eq.'s0 '.and.g3.eq.'s0 ')return
      endif

c QCD couplings
      if(smodel.eq.2.or.smodel.eq.12.or.smodel.eq.4.or.smodel.eq.34
     &    .or.smodel.eq.125)then
c 2 quarks - gluon
        if(g1.eq.'uq '.and.g2.eq.'uq~'.and.g3.eq.'gl ')return
        if(g1.eq.'dq '.and.g2.eq.'dq~'.and.g3.eq.'gl ')return
c 3 gluons
        if(g1.eq.'gl '.and.g2.eq.'gl '.and.g3.eq.'gl ')return
c 4 particle vertices
        if(schannel)then
          if(g1.eq.'gl '.and.g2.eq.'gl '.and.g3.eq.' 4 ')return
          if(g1.eq.'gl '.and.g2.eq.'gl '.and.g3.eq.' 4~')return
        endif
      endif

c SQCD couplings
      if(smodel.eq.4.or.smodel.eq.34)then
c gluino - quark - squark
        if(g1.eq.'uq '.and.g2.eq.'UQ~'.and.g3.eq.'GL ')return
        if(g1.eq.'uq~'.and.g2.eq.'UQ '.and.g3.eq.'GL ')return
        if(g1.eq.'dq '.and.g2.eq.'DQ~'.and.g3.eq.'GL ')return
        if(g1.eq.'dq~'.and.g2.eq.'DQ '.and.g3.eq.'GL ')return
c 2 gluinos - gluon
        if(g1.eq.'GL '.and.g2.eq.'GL '.and.g3.eq.'gl ')return
c 2 squarks - gluon
        if(g1.eq.'UQ '.and.g2.eq.'UQ~'.and.g3.eq.'gl ')return
        if(g1.eq.'DQ '.and.g2.eq.'DQ~'.and.g3.eq.'gl ')return
c 4 particle vertices
        if(g1.eq.'UQ '.and.g2.eq.'UQ~'.and.g3.eq.' 1 ')return
        if(g1.eq.'DQ '.and.g2.eq.'DQ~'.and.g3.eq.' 1 ')return
        if(g1.eq.'gl '.and.g2.eq.'gl '.and.g3.eq.' 1~')return
        if(g1.eq.'gl '.and.g2.eq.'ph '.and.g3.eq.' 1~')return
        if(g1.eq.'gl '.and.g2.eq.'Z0 '.and.g3.eq.' 1~')return
        if(g1.eq.'UQ '.and.g2.eq.'DQ~'.and.g3.eq.' 2 ')return
        if(g1.eq.'gl '.and.g2.eq.'W- '.and.g3.eq.' 2~')return
        if(g1.eq.'UQ~'.and.g2.eq.'DQ '.and.g3.eq.' 3~')return
        if(g1.eq.'gl '.and.g2.eq.'W-~'.and.g3.eq.' 3 ')return
      endif


      if(smodel.eq.3.or.smodel.eq.34)then
c 2 charginos - higgs
      if(g1.eq.'C- '.and.g2.eq.'C-~'.and.g3.eq.'h0 ')return
      if(g1.eq.'C- '.and.g2.eq.'C-~'.and.g3.eq.'H0 ')return
      if(g1.eq.'C- '.and.g2.eq.'C-~'.and.g3.eq.'A0 ')return
c 2 leptons - higgs
      if(light(i2).eq.0)then
c        if(g1.eq.'el '.and.g2.eq.'el~'.and.g3.eq.'h0 ')return
        if(g1.eq.'el '.and.g2.eq.'el~'.and.g3.eq.'H0 ')return
        if(g1.eq.'el '.and.g2.eq.'el~'.and.g3.eq.'A0 ')return
        if(g1.eq.'ne '.and.g2.eq.'el~'.and.g3.eq.'H- ')return
        if(g1.eq.'ne~'.and.g2.eq.'el '.and.g3.eq.'H-~')return
      endif
c 2 neutralinos - higgs
      if(g1.eq.'C0 '.and.g2.eq.'C0 '.and.g3.eq.'h0 ')return
      if(g1.eq.'C0 '.and.g2.eq.'C0 '.and.g3.eq.'H0 ')return
      if(g1.eq.'C0 '.and.g2.eq.'C0 '.and.g3.eq.'A0 ')return
c 2 quarks - higgs
      if(light(i1).eq.0.and.light(i2).eq.0)then
c        if(g1.eq.'dq '.and.g2.eq.'dq~'.and.g3.eq.'h0 ')return
c        if(g1.eq.'uq '.and.g2.eq.'uq~'.and.g3.eq.'h0 ')return
        if(g1.eq.'dq '.and.g2.eq.'dq~'.and.g3.eq.'H0 ')return
        if(g1.eq.'uq '.and.g2.eq.'uq~'.and.g3.eq.'H0 ')return
        if(g1.eq.'dq '.and.g2.eq.'dq~'.and.g3.eq.'A0 ')return
        if(g1.eq.'uq '.and.g2.eq.'uq~'.and.g3.eq.'A0 ')return
        if(g1.eq.'uq '.and.g2.eq.'dq~'.and.g3.eq.'H- ')return
        if(g1.eq.'uq~'.and.g2.eq.'dq '.and.g3.eq.'H-~')return
      endif
c chargino - lepton - slepton
      if(g1.eq.'ne '.and.g2.eq.'EL~'.and.g3.eq.'C- ')return
      if(g1.eq.'ne~'.and.g2.eq.'EL '.and.g3.eq.'C-~')return
      if(g1.eq.'NE '.and.g2.eq.'el~'.and.g3.eq.'C- ')return
      if(g1.eq.'NE~'.and.g2.eq.'el '.and.g3.eq.'C-~')return
c chargino - neutralino - higgs
      if(g1.eq.'C0 '.and.g2.eq.'C-~'.and.g3.eq.'H- ')return
      if(g1.eq.'C0 '.and.g2.eq.'C- '.and.g3.eq.'H-~')return
c chargino - quark - squark
      if(g1.eq.'uq '.and.g2.eq.'DQ~'.and.g3.eq.'C- ')return
      if(g1.eq.'uq~'.and.g2.eq.'DQ '.and.g3.eq.'C-~')return
      if(g1.eq.'UQ '.and.g2.eq.'dq~'.and.g3.eq.'C- ')return
      if(g1.eq.'UQ~'.and.g2.eq.'dq '.and.g3.eq.'C-~')return
c lepton - neutralino - slepton
      if(g1.eq.'ne '.and.g2.eq.'NE~'.and.g3.eq.'C0 ')return
      if(g1.eq.'ne~'.and.g2.eq.'NE '.and.g3.eq.'C0 ')return
      if(g1.eq.'el '.and.g2.eq.'EL~'.and.g3.eq.'C0 ')return
      if(g1.eq.'el~'.and.g2.eq.'EL '.and.g3.eq.'C0 ')return
c neutralino - quark - squark
      if(g1.eq.'uq '.and.g2.eq.'UQ~'.and.g3.eq.'C0 ')return
      if(g1.eq.'uq~'.and.g2.eq.'UQ '.and.g3.eq.'C0 ')return
      if(g1.eq.'dq '.and.g2.eq.'DQ~'.and.g3.eq.'C0 ')return
      if(g1.eq.'dq~'.and.g2.eq.'DQ '.and.g3.eq.'C0 ')return
c 2 charginos - gauge boson
      if(g1.eq.'C- '.and.g2.eq.'C-~'.and.g3.eq.'ph ')return
      if(g1.eq.'C- '.and.g2.eq.'C-~'.and.g3.eq.'Z0 ')return
c 2 leptons - gauge boson
c      if(g1.eq.'el '.and.g2.eq.'el~'.and.g3.eq.'ph ')return
c      if(g1.eq.'ne '.and.g2.eq.'ne~'.and.g3.eq.'Z0 ')return
c      if(g1.eq.'el '.and.g2.eq.'el~'.and.g3.eq.'Z0 ')return
c      if(g1.eq.'ne '.and.g2.eq.'el~'.and.g3.eq.'W- ')return
c      if(g1.eq.'ne~'.and.g2.eq.'el '.and.g3.eq.'W-~')return
c 2 neutralinos - gauge bosons
      if(g1.eq.'C0 '.and.g2.eq.'C0 '.and.g3.eq.'Z0 ')return
c 2 quarks - gauge boson
c      if(g1.eq.'uq '.and.g2.eq.'uq~'.and.g3.eq.'ph ')return
c      if(g1.eq.'dq '.and.g2.eq.'dq~'.and.g3.eq.'ph ')return
c      if(g1.eq.'uq '.and.g2.eq.'uq~'.and.g3.eq.'Z0 ')return
c      if(g1.eq.'dq '.and.g2.eq.'dq~'.and.g3.eq.'Z0 ')return
c      if(g1.eq.'uq '.and.g2.eq.'dq~'.and.g3.eq.'W- ')return
c      if(g1.eq.'uq~'.and.g2.eq.'dq '.and.g3.eq.'W-~')return
c chargino - neutralino - gauge boson
      if(g1.eq.'C0 '.and.g2.eq.'C-~'.and.g3.eq.'W- ')return
      if(g1.eq.'C0 '.and.g2.eq.'C- '.and.g3.eq.'W-~')return
c 3 higgs
c      if(g1.eq.'h0 '.and.g2.eq.'h0 '.and.g3.eq.'h0 ')return
      if(g1.eq.'h0 '.and.g2.eq.'h0 '.and.g3.eq.'H0 ')return
      if(g1.eq.'h0 '.and.g2.eq.'H0 '.and.g3.eq.'H0 ')return
      if(g1.eq.'H0 '.and.g2.eq.'H0 '.and.g3.eq.'H0 ')return
      if(g1.eq.'h0 '.and.g2.eq.'A0 '.and.g3.eq.'A0 ')return
      if(g1.eq.'H0 '.and.g2.eq.'A0 '.and.g3.eq.'A0 ')return
      if(g1.eq.'h0 '.and.g2.eq.'H-~'.and.g3.eq.'H- ')return
      if(g1.eq.'H0 '.and.g2.eq.'H-~'.and.g3.eq.'H- ')return
c higgs - 2 sleptons
      if(light(i2).eq.0)then
        if(g1.eq.'NE '.and.g2.eq.'NE~'.and.g3.eq.'h0 ')return
        if(g1.eq.'NE '.and.g2.eq.'NE~'.and.g3.eq.'H0 ')return
        if(g1.eq.'EL '.and.g2.eq.'EL~'.and.g3.eq.'h0 ')return
        if(g1.eq.'EL '.and.g2.eq.'EL~'.and.g3.eq.'H0 ')return
        if(g1.eq.'EL '.and.g2.eq.'EL~'.and.g3.eq.'A0 ')return
        if(g1.eq.'NE '.and.g2.eq.'EL~'.and.g3.eq.'H- ')return
        if(g1.eq.'NE~'.and.g2.eq.'EL '.and.g3.eq.'H-~')return
      endif
c higgs - 2 squarks
      if(light(i1).eq.0.and.light(i2).eq.0)then
        if(g1.eq.'UQ '.and.g2.eq.'UQ~'.and.g3.eq.'h0 ')return
        if(g1.eq.'UQ '.and.g2.eq.'UQ~'.and.g3.eq.'H0 ')return
        if(g1.eq.'UQ '.and.g2.eq.'UQ~'.and.g3.eq.'A0 ')return
        if(g1.eq.'DQ '.and.g2.eq.'DQ~'.and.g3.eq.'h0 ')return
        if(g1.eq.'DQ '.and.g2.eq.'DQ~'.and.g3.eq.'H0 ')return
        if(g1.eq.'DQ '.and.g2.eq.'DQ~'.and.g3.eq.'A0 ')return
        if(g1.eq.'UQ '.and.g2.eq.'DQ~'.and.g3.eq.'H- ')return
        if(g1.eq.'UQ~'.and.g2.eq.'DQ '.and.g3.eq.'H-~')return
      endif
c 2 higgs - gauge boson
      if(g1.eq.'h0 '.and.g2.eq.'A0 '.and.g3.eq.'Z0 ')return
      if(g1.eq.'H0 '.and.g2.eq.'A0 '.and.g3.eq.'Z0 ')return
      if(g1.eq.'H- '.and.g2.eq.'H-~'.and.g3.eq.'ph ')return
      if(g1.eq.'H- '.and.g2.eq.'H-~'.and.g3.eq.'Z0 ')return
      if(g1.eq.'h0 '.and.g2.eq.'H-~'.and.g3.eq.'W- ')return
      if(g1.eq.'h0 '.and.g2.eq.'H- '.and.g3.eq.'W-~')return
      if(g1.eq.'H0 '.and.g2.eq.'H-~'.and.g3.eq.'W- ')return
      if(g1.eq.'H0 '.and.g2.eq.'H- '.and.g3.eq.'W-~')return
      if(g1.eq.'A0 '.and.g2.eq.'H-~'.and.g3.eq.'W- ')return
      if(g1.eq.'A0 '.and.g2.eq.'H- '.and.g3.eq.'W-~')return
c 2 sleptons - gauge boson
      if(g1.eq.'EL '.and.g2.eq.'EL~'.and.g3.eq.'ph ')return
      if(g1.eq.'NE '.and.g2.eq.'NE~'.and.g3.eq.'Z0 ')return
      if(g1.eq.'EL '.and.g2.eq.'EL~'.and.g3.eq.'Z0 ')return
      if(g1.eq.'NE '.and.g2.eq.'EL~'.and.g3.eq.'W- ')return
      if(g1.eq.'NE~'.and.g2.eq.'EL '.and.g3.eq.'W-~')return
c 2 squarks - gauge boson
      if(g1.eq.'UQ '.and.g2.eq.'UQ~'.and.g3.eq.'ph ')return
      if(g1.eq.'DQ '.and.g2.eq.'DQ~'.and.g3.eq.'ph ')return
      if(g1.eq.'UQ '.and.g2.eq.'UQ~'.and.g3.eq.'Z0 ')return
      if(g1.eq.'DQ '.and.g2.eq.'DQ~'.and.g3.eq.'Z0 ')return
      if(g1.eq.'UQ '.and.g2.eq.'DQ~'.and.g3.eq.'W- ')return
      if(g1.eq.'UQ~'.and.g2.eq.'DQ '.and.g3.eq.'W-~')return
c higgs - 2 gauge bosons
c      if(g1.eq.'h0 '.and.g2.eq.'Z0 '.and.g3.eq.'Z0 ')return
      if(g1.eq.'H0 '.and.g2.eq.'Z0 '.and.g3.eq.'Z0 ')return
c      if(g1.eq.'h0 '.and.g2.eq.'W- '.and.g3.eq.'W-~')return
      if(g1.eq.'H0 '.and.g2.eq.'W- '.and.g3.eq.'W-~')return
c 3 gauge bosons
c      if(g1.eq.'ph '.and.g2.eq.'W- '.and.g3.eq.'W-~')return
c      if(g1.eq.'Z0 '.and.g2.eq.'W- '.and.g3.eq.'W-~')return
c gluino - quark - squark
c      if(g1.eq.'uq '.and.g2.eq.'UQ~'.and.g3.eq.'GL ')return
c      if(g1.eq.'uq~'.and.g2.eq.'UQ '.and.g3.eq.'GL ')return
c      if(g1.eq.'dq '.and.g2.eq.'DQ~'.and.g3.eq.'GL ')return
c      if(g1.eq.'dq~'.and.g2.eq.'DQ '.and.g3.eq.'GL ')return
c 2 gluinos - gluon
c      if(g1.eq.'GL '.and.g2.eq.'GL '.and.g3.eq.'gl ')return
c 2 quarks - gluon
c      if(g1.eq.'uq '.and.g2.eq.'uq~'.and.g3.eq.'gl ')return
c      if(g1.eq.'dq '.and.g2.eq.'dq~'.and.g3.eq.'gl ')return
c 2 squarks - gluon
c      if(g1.eq.'UQ '.and.g2.eq.'UQ~'.and.g3.eq.'gl ')return
c      if(g1.eq.'DQ '.and.g2.eq.'DQ~'.and.g3.eq.'gl ')return
c 3 gluons
c      if(g1.eq.'gl '.and.g2.eq.'gl '.and.g3.eq.'gl ')return
c 4 particle vertices
c      if(g1.eq.'UQ '.and.g2.eq.'UQ~'.and.g3.eq.' 1 ')return
c      if(g1.eq.'DQ '.and.g2.eq.'DQ~'.and.g3.eq.' 1 ')return
c      if(g1.eq.'gl '.and.g2.eq.'gl '.and.g3.eq.' 1~')return
c      if(g1.eq.'gl '.and.g2.eq.'ph '.and.g3.eq.' 1~')return
c      if(g1.eq.'gl '.and.g2.eq.'Z0 '.and.g3.eq.' 1~')return
c      if(g1.eq.'UQ '.and.g2.eq.'DQ~'.and.g3.eq.' 2 ')return
c      if(g1.eq.'gl '.and.g2.eq.'W- '.and.g3.eq.' 2~')return
c      if(g1.eq.'UQ~'.and.g2.eq.'DQ '.and.g3.eq.' 3~')return
c      if(g1.eq.'gl '.and.g2.eq.'W-~'.and.g3.eq.' 3 ')return
c      if(schannel)then
c        if(g1.eq.'gl '.and.g2.eq.'gl '.and.g3.eq.' 4 ')return
c        if(g1.eq.'gl '.and.g2.eq.'gl '.and.g3.eq.' 4~')return
c      endif
      if(g1.eq.'NE '.and.g2.eq.'EL~'.and.g3.eq.' 5 ')return
      if(g1.eq.'UQ '.and.g2.eq.'DQ~'.and.g3.eq.' 5 ')return
c      if(g1.eq.'ph '.and.g2.eq.'W-~'.and.g3.eq.' 5 ')return
c      if(g1.eq.'Z0 '.and.g2.eq.'W-~'.and.g3.eq.' 5 ')return
c      if(g1.eq.'ph '.and.g2.eq.'W- '.and.g3.eq.' 5~')return
c      if(g1.eq.'Z0 '.and.g2.eq.'W- '.and.g3.eq.' 5~')return
      if(g1.eq.'NE~'.and.g2.eq.'EL '.and.g3.eq.' 6~')return
      if(g1.eq.'UQ~'.and.g2.eq.'DQ '.and.g3.eq.' 6~')return
c      if(g1.eq.'ph '.and.g2.eq.'W- '.and.g3.eq.' 6~')return
c      if(g1.eq.'Z0 '.and.g2.eq.'W- '.and.g3.eq.' 6~')return
c      if(g1.eq.'ph~'.and.g2.eq.'W-~'.and.g3.eq.' 6 ')return
c      if(g1.eq.'Z0 '.and.g2.eq.'W-~'.and.g3.eq.' 6 ')return
      if(g1.eq.'H+ '.and.g2.eq.'H- '.and.g3.eq.' 7 ')return
c      if(g1.eq.'W+ '.and.g2.eq.'W- '.and.g3.eq.' 7 ')return
      if(g1.eq.'EL '.and.g2.eq.'EL~'.and.g3.eq.' 7 ')return
      if(g1.eq.'UQ '.and.g2.eq.'UQ~'.and.g3.eq.' 7 ')return
      if(g1.eq.'DQ '.and.g2.eq.'DQ~'.and.g3.eq.' 7 ')return
      if(g1.eq.'H+ '.and.g2.eq.'H- '.and.g3.eq.' 7~')return
c      if(g1.eq.'W+ '.and.g2.eq.'W- '.and.g3.eq.' 7~')return
      if(g1.eq.'EL '.and.g2.eq.'EL~'.and.g3.eq.' 7~')return
      if(g1.eq.'UQ '.and.g2.eq.'UQ~'.and.g3.eq.' 7~')return
      if(g1.eq.'DQ '.and.g2.eq.'DQ~'.and.g3.eq.' 7~')return
c      if(g1.eq.'h0 '.and.g2.eq.'h0 '.and.g3.eq.' 7~')return
      if(g1.eq.'h0 '.and.g2.eq.'H0 '.and.g3.eq.' 7~')return
      if(g1.eq.'H0 '.and.g2.eq.'H0 '.and.g3.eq.' 7~')return
      if(g1.eq.'A0 '.and.g2.eq.'A0 '.and.g3.eq.' 7~')return
c      if(g1.eq.'ph '.and.g2.eq.'ph '.and.g3.eq.' 7~')return
c      if(g1.eq.'ph '.and.g2.eq.'Z0 '.and.g3.eq.' 7~')return
c      if(g1.eq.'Z0 '.and.g2.eq.'Z0 '.and.g3.eq.' 7~')return
      if(g1.eq.'NE '.and.g2.eq.'NE~'.and.g3.eq.' 7~')return
      if(schannel)then
c        if(g1.eq.'h0 '.and.g2.eq.'h0 '.and.g3.eq.' 8 ')return
c        if(g1.eq.'h0 '.and.g2.eq.'h0 '.and.g3.eq.' 8~')return
        if(g1.eq.'H0 '.and.g2.eq.'H0 '.and.g3.eq.' 9 ')return
        if(g1.eq.'H0 '.and.g2.eq.'H0 '.and.g3.eq.' 9~')return
        if(g1.eq.'A0 '.and.g2.eq.'A0 '.and.g3.eq.'10 ')return
        if(g1.eq.'A0 '.and.g2.eq.'A0 '.and.g3.eq.'10~')return
      endif
      if(g1.eq.'NE '.and.g2.eq.'NE~'.and.g3.eq.'11 ')return
      if(g1.eq.'ph '.and.g2.eq.'ph '.and.g3.eq.'11 ')return
      if(g1.eq.'h0 '.and.g2.eq.'h0 '.and.g3.eq.'11 ')return
      if(g1.eq.'h0 '.and.g2.eq.'H0 '.and.g3.eq.'11~')return
      if(g1.eq.'H0 '.and.g2.eq.'H0 '.and.g3.eq.'11~')return
      if(g1.eq.'A0 '.and.g2.eq.'A0 '.and.g3.eq.'11~')return
      if(g1.eq.'NE '.and.g2.eq.'NE~'.and.g3.eq.'11~')return
      if(g1.eq.'h0 '.and.g2.eq.'H0 '.and.g3.eq.'12 ')return
      if(g1.eq.'H0 '.and.g2.eq.'H0 '.and.g3.eq.'12~')return
      if(g1.eq.'A0 '.and.g2.eq.'A0 '.and.g3.eq.'12~')return
      if(g1.eq.'NE '.and.g2.eq.'NE~'.and.g3.eq.'12~')return
      if(g1.eq.'Z0 '.and.g2.eq.'Z0 '.and.g3.eq.'12~')return
      if(g1.eq.'H0 '.and.g2.eq.'H0 '.and.g3.eq.'13 ')return
      if(g1.eq.'A0 '.and.g2.eq.'A0 '.and.g3.eq.'13~')return
      if(g1.eq.'NE '.and.g2.eq.'NE~'.and.g3.eq.'13~')return
      if(g1.eq.'Z0 '.and.g2.eq.'Z0 '.and.g3.eq.'13~')return
      if(g1.eq.'A0 '.and.g2.eq.'A0 '.and.g3.eq.'14 ')return
      if(g1.eq.'NE '.and.g2.eq.'NE~'.and.g3.eq.'14~')return
      if(g1.eq.'Z0 '.and.g2.eq.'Z0 '.and.g3.eq.'14~')return
      if(g1.eq.'NE '.and.g2.eq.'NE~'.and.g3.eq.'15 ')return
      if(g1.eq.'Z0 '.and.g2.eq.'Z0 '.and.g3.eq.'15~')return
      endif


c add non-standard couplings (with family conservation!)
c     if(nonstandardcoup(p1,p2,p3,g1,g2,g3,schannel))return

      vertexg=.false.
      end
